@@ -3,9 +3,9 @@
 Limitations:
 
 - When generating examples for use as doctests, make sure that you have
-  pretty-printing OFF.  This can be done either by starting ipython with the
-  flag '--nopprint', by setting pprint to 0 in your ipythonrc file, or by
-  interactively disabling it with %Pprint.  This is required so that IPython
+  pretty-printing OFF.  This can be done either by setting the
+  ``PlainTextFormatter.pprint`` option in your configuration file to  False, or
+  by interactively disabling it with  %Pprint.  This is required so that IPython
   output matches that of normal Python, which is used by doctest for internal
   execution.
 
@@ -19,7 +19,7 @@ Limitations:
 # Module imports
 
 # From the standard library
-import __builtin__
+import __builtin__ as builtin_mod
 import commands
 import doctest
 import inspect
@@ -49,182 +49,14 @@ from nose.util import anyp, getpackage, test_address, resolve_name, tolist
 
 #-----------------------------------------------------------------------------
 # Module globals and other constants
+#-----------------------------------------------------------------------------
 
 log = logging.getLogger(__name__)
 
-###########################################################################
-# *** HACK ***
-# We must start our own ipython object and heavily muck with it so that all the
-# modifications IPython makes to system behavior don't send the doctest
-# machinery into a fit.  This code should be considered a gross hack, but it
-# gets the job done.
-
-def default_argv():
-    """Return a valid default argv for creating testing instances of ipython"""
-
-    # Get the install directory for the user configuration and tell ipython to
-    # use the default profile from there.
-    from IPython import UserConfig
-    ipcdir = os.path.dirname(UserConfig.__file__)
-    #ipconf = os.path.join(ipcdir,'ipy_user_conf.py')
-    ipconf = os.path.join(ipcdir,'ipythonrc')
-    #print 'conf:',ipconf # dbg
-    
-    return ['--colors=NoColor','--noterm_title','-rcfile=%s' % ipconf]
-
-
-# Hack to modify the %run command so we can sync the user's namespace with the
-# test globals.  Once we move over to a clean magic system, this will be done
-# with much less ugliness.
-
-class py_file_finder(object):
-    def __init__(self,test_filename):
-        self.test_filename = test_filename
-        
-    def __call__(self,name):
-        from IPython.genutils import get_py_filename
-        try:
-            return get_py_filename(name)
-        except IOError:
-            test_dir = os.path.dirname(self.test_filename)
-            new_path = os.path.join(test_dir,name)
-            return get_py_filename(new_path)
-    
-
-def _run_ns_sync(self,arg_s,runner=None):
-    """Modified version of %run that syncs testing namespaces.
-
-    This is strictly needed for running doctests that call %run.
-    """
-
-    # When tests call %run directly (not via doctest) these function attributes
-    # are not set
-    try:
-        fname = _run_ns_sync.test_filename
-    except AttributeError:
-        fname = arg_s
-
-    finder = py_file_finder(fname)
-    out = _ip.IP.magic_run_ori(arg_s,runner,finder)
-    
-    # Simliarly, there is no test_globs when a test is NOT a doctest
-    if hasattr(_run_ns_sync,'test_globs'):
-        _run_ns_sync.test_globs.update(_ip.user_ns)
-    return out
-
-
-class ipnsdict(dict):
-    """A special subclass of dict for use as an IPython namespace in doctests.
-
-    This subclass adds a simple checkpointing capability so that when testing
-    machinery clears it (we use it as the test execution context), it doesn't
-    get completely destroyed.
-    """
-    
-    def __init__(self,*a):
-        dict.__init__(self,*a)
-        self._savedict = {}
-        
-    def clear(self):
-        dict.clear(self)
-        self.update(self._savedict)
-
-    def _checkpoint(self):
-        self._savedict.clear()
-        self._savedict.update(self)
-
-    def update(self,other):
-        self._checkpoint()
-        dict.update(self,other)
-
-        # If '_' is in the namespace, python won't set it when executing code,
-        # and we have examples that test it.  So we ensure that the namespace
-        # is always 'clean' of it before it's used for test code execution.
-        self.pop('_',None)
-
-        # The builtins namespace must *always* be the real __builtin__ module,
-        # else weird stuff happens.  The main ipython code does have provisions
-        # to ensure this after %run, but since in this class we do some
-        # aggressive low-level cleaning of the execution namespace, we need to
-        # correct for that ourselves, to ensure consitency with the 'real'
-        # ipython.
-        self['__builtins__'] = __builtin__
-        
-
-def start_ipython():
-    """Start a global IPython shell, which we need for IPython-specific syntax.
-    """
-
-    # This function should only ever run once!
-    if hasattr(start_ipython,'already_called'):
-        return
-    start_ipython.already_called = True
-
-    # Ok,  first time we're called, go ahead
-    import new
-
-    import IPython
-
-    def xsys(cmd):
-        """Execute a command and print its output.
-
-        This is just a convenience function to replace the IPython system call
-        with one that is more doctest-friendly.
-        """
-        cmd = _ip.IP.var_expand(cmd,depth=1)
-        sys.stdout.write(commands.getoutput(cmd))
-        sys.stdout.flush()
-
-    # Store certain global objects that IPython modifies
-    _displayhook = sys.displayhook
-    _excepthook = sys.excepthook
-    _main = sys.modules.get('__main__')
-
-    argv = default_argv()
-    
-    # Start IPython instance.  We customize it to start with minimal frills.
-    user_ns,global_ns = IPython.ipapi.make_user_namespaces(ipnsdict(),dict())
-    IPython.Shell.IPShell(argv,user_ns,global_ns)
-
-    # Deactivate the various python system hooks added by ipython for
-    # interactive convenience so we don't confuse the doctest system
-    sys.modules['__main__'] = _main
-    sys.displayhook = _displayhook
-    sys.excepthook = _excepthook
-
-    # So that ipython magics and aliases can be doctested (they work by making
-    # a call into a global _ip object)
-    _ip = IPython.ipapi.get()
-    __builtin__._ip = _ip
-
-    # Modify the IPython system call with one that uses getoutput, so that we
-    # can capture subcommands and print them to Python's stdout, otherwise the
-    # doctest machinery would miss them.
-    _ip.system = xsys
-
-    # Also patch our %run function in.
-    im = new.instancemethod(_run_ns_sync,_ip.IP, _ip.IP.__class__)
-    _ip.IP.magic_run_ori = _ip.IP.magic_run
-    _ip.IP.magic_run = im
-
-    # XXX - For some very bizarre reason, the loading of %history by default is
-    # failing.  This needs to be fixed later, but for now at least this ensures
-    # that tests that use %hist run to completion.
-    from IPython import history
-    history.init_ipython(_ip)
-    if not hasattr(_ip.IP,'magic_history'):
-        raise RuntimeError("Can't load magics, aborting")
-
-
-# The start call MUST be made here.  I'm not sure yet why it doesn't work if
-# it is made later, at plugin initialization time, but in all my tests, that's
-# the case.
-start_ipython()
-
-# *** END HACK ***
-###########################################################################
 
+#-----------------------------------------------------------------------------
 # Classes and functions
+#-----------------------------------------------------------------------------
 
 def is_extension_module(filename):
     """Return whether the given filename is an extension module.
@@ -236,7 +68,7 @@ def is_extension_module(filename):
 
 class DocTestSkip(object):
     """Object wrapper for doctests to be skipped."""
-    
+
     ds_skip = """Doctest to skip.
     >>> 1 #doctest: +SKIP
     """
@@ -287,7 +119,7 @@ class DocTestFinder(doctest.DocTestFinder):
         Find tests for the given object and any contained objects, and
         add them to `tests`.
         """
-
+        #print '_find for:', obj, name, module  # dbg
         if hasattr(obj,"skip_doctest"):
             #print 'SKIPPING DOCTEST FOR:',obj  # dbg
             obj = DocTestSkip(obj)
@@ -334,13 +166,13 @@ class DocTestFinder(doctest.DocTestFinder):
 
 class IPDoctestOutputChecker(doctest.OutputChecker):
     """Second-chance checker with support for random tests.
-    
+
     If the default comparison doesn't pass, this checker looks in the expected
     output string for flags that tell us to ignore the output.
     """
 
     random_re = re.compile(r'#\s*random\s+')
-    
+
     def check_output(self, want, got, optionflags):
         """Check output, accepting special markers embedded in the output.
 
@@ -386,6 +218,7 @@ class DocTestCase(doctests.DocTestCase):
         self._dt_optionflags = optionflags
         self._dt_checker = checker
         self._dt_test = test
+        self._dt_test_globs_ori = test.globs
         self._dt_setUp = setUp
         self._dt_tearDown = tearDown
 
@@ -395,14 +228,15 @@ class DocTestCase(doctests.DocTestCase):
         self._dt_runner = runner
 
 
-        # Each doctest should remember what directory it was loaded from...
-        self._ori_dir = os.getcwd()
+        # Each doctest should remember the directory it was loaded from, so
+        # things like %run work without too many contortions
+        self._ori_dir = os.path.dirname(test.filename)
 
     # Modified runTest from the default stdlib
     def runTest(self):
         test = self._dt_test
         runner = self._dt_runner
-        
+
         old = sys.stdout
         new = StringIO()
         optionflags = self._dt_optionflags
@@ -416,7 +250,8 @@ class DocTestCase(doctests.DocTestCase):
             # Save our current directory and switch out to the one where the
             # test was originally created, in case another doctest did a
             # directory change.  We'll restore this in the finally clause.
-            curdir = os.getcwd()
+            curdir = os.getcwdu()
+            #print 'runTest in dir:', self._ori_dir  # dbg
             os.chdir(self._ori_dir)
 
             runner.DIVIDER = "-"*70
@@ -431,17 +266,31 @@ class DocTestCase(doctests.DocTestCase):
 
     def setUp(self):
         """Modified test setup that syncs with ipython namespace"""
-        
-        if isinstance(self._dt_test.examples[0],IPExample):
+        #print "setUp test", self._dt_test.examples # dbg
+        if isinstance(self._dt_test.examples[0], IPExample):
             # for IPython examples *only*, we swap the globals with the ipython
             # namespace, after updating it with the globals (which doctest
             # fills with the necessary info from the module being tested).
-            _ip.IP.user_ns.update(self._dt_test.globs)
-            self._dt_test.globs = _ip.IP.user_ns
+            self.user_ns_orig = {}
+            self.user_ns_orig.update(_ip.user_ns)
+            _ip.user_ns.update(self._dt_test.globs)
+            # We must remove the _ key in the namespace, so that Python's
+            # doctest code sets it naturally
+            _ip.user_ns.pop('_', None)
+            _ip.user_ns['__builtins__'] = builtin_mod
+            self._dt_test.globs = _ip.user_ns
 
         super(DocTestCase, self).setUp()
 
     def tearDown(self):
+
+        # Undo the test.globs reassignment we made, so that the parent class
+        # teardown doesn't destroy the ipython namespace
+        if isinstance(self._dt_test.examples[0], IPExample):
+            self._dt_test.globs = self._dt_test_globs_ori
+            _ip.user_ns.clear()
+            _ip.user_ns.update(self.user_ns_orig)
+
         # XXX - fperez: I am not sure if this is truly a bug in nose 0.11, but
         # it does look like one to me: its tearDown method tries to run
         #
@@ -528,7 +377,7 @@ class IPDocTestParser(doctest.DocTestParser):
 
     # Mark tests to be executed in an external process - currently unsupported.
     _EXTERNAL_IP = re.compile(r'#\s*ipdoctest:\s*EXTERNAL')
-    
+
     def ip2py(self,source):
         """Convert input IPython source into valid Python."""
         out = []
@@ -538,7 +387,7 @@ class IPDocTestParser(doctest.DocTestParser):
         # and turned into lines, so it looks to the parser like regular user
         # input
         for lnum,line in enumerate(source.strip().splitlines()):
-            newline(_ip.IP.prefilter(line,lnum>0))
+            newline(_ip.prefilter(line,lnum>0))
         newline('')  # ensure a closing newline, needed by doctest
         #print "PYSRC:", '\n'.join(out)  # dbg
         return '\n'.join(out)
@@ -611,7 +460,7 @@ class IPDocTestParser(doctest.DocTestParser):
             # Append the random-output marker (it defaults to empty in most
             # cases, it's only non-empty for 'all-random' tests):
             want += random_marker
-                
+
             if Example is IPExternalExample:
                 options[doctest.NORMALIZE_WHITESPACE] = True
                 want += '\n'
@@ -721,7 +570,7 @@ SKIP = doctest.register_optionflag('SKIP')
 class IPDocTestRunner(doctest.DocTestRunner,object):
     """Test runner that synchronizes the IPython namespace with test globals.
     """
-    
+
     def run(self, test, compileflags=None, out=None, clear_globs=True):
 
         # Hack: ipython needs access to the execution context of the example,
@@ -730,9 +579,11 @@ class IPDocTestRunner(doctest.DocTestRunner,object):
         # attribute.  Our new %run will then only make the namespace update
         # when called (rather than unconconditionally updating test.globs here
         # for all examples, most of which won't be calling %run anyway).
-        _run_ns_sync.test_globs = test.globs
-        _run_ns_sync.test_filename = test.filename
-        
+        #_ip._ipdoctest_test_globs = test.globs
+        #_ip._ipdoctest_test_filename = test.filename
+
+        test.globs.update(_ip.user_ns)
+
         return super(IPDocTestRunner,self).run(test,
                                                compileflags,out,clear_globs)
 
@@ -790,6 +641,9 @@ class ExtensionDoctest(doctests.Doctest):
 
     def configure(self, options, config):
         Plugin.configure(self, options, config)
+        # Pull standard doctest plugin out of config; we will do doctesting
+        config.plugins.plugins = [p for p in config.plugins.plugins
+                                  if p.name != 'doctest']
         self.doctest_tests = options.doctest_tests
         self.extension = tolist(options.doctestExtension)
 
@@ -816,7 +670,7 @@ class ExtensionDoctest(doctests.Doctest):
 
     def loadTestsFromModule(self, module):
         #print '*** ipdoctest - lTM',module  # dbg
-        
+
         if not self.matches(module.__name__):
             log.debug("Doctest doesn't want module %s", module)
             return
@@ -838,13 +692,14 @@ class ExtensionDoctest(doctests.Doctest):
                 continue
             if not test.filename:
                 test.filename = module_file
-            
+
             yield DocTestCase(test,
                               optionflags=optionflags,
                               checker=self.checker)
 
 
     def loadTestsFromFile(self, filename):
+        #print "ipdoctest - from file", filename # dbg
         if is_extension_module(filename):
             for t in self.loadTestsFromExtensionModule(filename):
                 yield t
@@ -871,7 +726,7 @@ class ExtensionDoctest(doctests.Doctest):
         Modified version that accepts extension modules as valid containers for
         doctests.
         """
-        # print '*** ipdoctest- wantFile:',filename  # dbg
+        #print '*** ipdoctest- wantFile:',filename  # dbg
 
         for pat in self.exclude_patterns:
             if pat.search(filename):
@@ -883,17 +738,29 @@ class ExtensionDoctest(doctests.Doctest):
         else:
             return doctests.Doctest.wantFile(self,filename)
 
+    def wantDirectory(self, directory):
+        """Return whether the given directory should be scanned for tests.
+
+        Modified version that supports exclusions.
+        """
+
+        for pat in self.exclude_patterns:
+            if pat.search(directory):
+                return False
+        return True
+
 
 class IPythonDoctest(ExtensionDoctest):
     """Nose Plugin that supports doctests in extension modules.
     """
     name = 'ipdoctest'   # call nosetests with --with-ipdoctest
     enabled = True
-    
+
     def makeTest(self, obj, parent):
         """Look for doctests in the given object, which will be a
         function, method or class.
         """
+        #print 'Plugin analyzing:', obj, parent  # dbg
         # always use whitespace and ellipsis options
         optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
 
@@ -908,6 +775,7 @@ class IPythonDoctest(ExtensionDoctest):
                                   checker=self.checker)
 
     def options(self, parser, env=os.environ):
+        #print "Options for nose plugin:", self.name # dbg
         Plugin.options(self, parser, env)
         parser.add_option('--ipdoctest-tests', action='store_true',
                           dest='ipdoctest_tests',
@@ -928,7 +796,11 @@ class IPythonDoctest(ExtensionDoctest):
             parser.set_defaults(ipdoctest_extension=tolist(env_setting))
 
     def configure(self, options, config):
+        #print "Configuring nose plugin:", self.name # dbg
         Plugin.configure(self, options, config)
+        # Pull standard doctest plugin out of config; we will do doctesting
+        config.plugins.plugins = [p for p in config.plugins.plugins
+                                  if p.name != 'doctest']
         self.doctest_tests = options.ipdoctest_tests
         self.extension = tolist(options.ipdoctest_extension)
 
