@@ -14,6 +14,7 @@
 
 import os
 import math
+import random
 
 import nose.tools as nt
 
@@ -32,13 +33,37 @@ def test_columnize():
     items = [l*size for l in 'abc']
     out = text.columnize(items, displaywidth=80)
     nt.assert_equals(out, 'aaaaa  bbbbb  ccccc\n')
-    out = text.columnize(items, displaywidth=10)
+    out = text.columnize(items, displaywidth=12)
     nt.assert_equals(out, 'aaaaa  ccccc\nbbbbb\n')
+    out = text.columnize(items, displaywidth=10)
+    nt.assert_equals(out, 'aaaaa\nbbbbb\nccccc\n')
 
+def test_columnize_random():
+    """Test with random input to hopfully catch edge case """
+    for nitems in [random.randint(2,70) for i in range(2,20)]:
+        displaywidth = random.randint(20,200)
+        rand_len = [random.randint(2,displaywidth) for i in range(nitems)]
+        items = ['x'*l for l in rand_len]
+        out = text.columnize(items, displaywidth=displaywidth)
+        longer_line = max([len(x) for x in out.split('\n')])
+        longer_element = max(rand_len)
+        if longer_line > displaywidth:
+            print "Columnize displayed something lager than displaywidth : %s " % longer_line
+            print "longer element : %s " % longer_element
+            print "displaywidth : %s " % displaywidth
+            print "number of element : %s " % nitems
+            print "size of each element :\n %s" % rand_len
+            assert False
+
+def test_columnize_medium():
+    """Test with inputs than shouldn't be wider tahn 80 """
+    size = 40
+    items = [l*size for l in 'abc']
+    out = text.columnize(items, displaywidth=80)
+    nt.assert_equals(out, '\n'.join(items+['']))
 
 def test_columnize_long():
     """Test columnize with inputs longer than the display window"""
-    text.columnize(['a'*81, 'b'*81], displaywidth=80)
     size = 11
     items = [l*size for l in 'abc']
     out = text.columnize(items, displaywidth=size-1)
@@ -113,3 +138,33 @@ def test_dollar_formatter():
     nt.assert_equals(s, "12 $HOME")
     s = f.format("${foo}", foo="HOME")
     nt.assert_equals(s, "$HOME")
+
+
+def test_long_substr():
+    data = ['hi']
+    nt.assert_equals(text.long_substr(data), 'hi')
+
+
+def test_long_substr2():
+    data = ['abc', 'abd', 'abf', 'ab']
+    nt.assert_equals(text.long_substr(data), 'ab')
+
+
+def test_strip_email():
+    src = """\
+        >> >>> def f(x):
+        >> ...   return x+1
+        >> ... 
+        >> >>> zz = f(2.5)"""
+    cln = """\
+>>> def f(x):
+...   return x+1
+... 
+>>> zz = f(2.5)"""
+    nt.assert_equals(text.strip_email_quotes(src), cln)
+
+
+def test_strip_email2():
+    src = '> > > list()'
+    cln = 'list()'
+    nt.assert_equals(text.strip_email_quotes(src), cln)
