@@ -44,6 +44,9 @@ def oscmd(s):
     print(">", s)
     os.system(s)
 
+# Py3 compatibility hacks, without assuming IPython itself is installed with
+# the full py3compat machinery.
+
 try:
     execfile
 except NameError:
@@ -97,11 +100,11 @@ def find_packages():
     """
     Find all of IPython's packages.
     """
-    excludes = ['deathrow']
+    excludes = ['deathrow', 'quarantine']
     packages = []
     for dir,subdirs,files in os.walk('IPython'):
         package = dir.replace(os.path.sep, '.')
-        if any([ package.startswith('IPython.'+exc) for exc in excludes ]):
+        if any(package.startswith('IPython.'+exc) for exc in excludes):
             # package is to be excluded (e.g. deathrow)
             continue
         if '__init__.py' not in files:
@@ -142,6 +145,7 @@ def find_package_data():
     package_data = {
         'IPython.config.profile' : ['README*', '*/*.py'],
         'IPython.testing' : ['*.txt'],
+        'IPython.testing.plugin' : ['*.txt'],
         'IPython.frontend.html.notebook' : ['templates/*'] + static_data,
         'IPython.frontend.qt.console' : ['resources/icon/*.svg'],
     }
@@ -251,6 +255,44 @@ def make_man_update_target(manpage):
                locals() )
     return (manpath_gz, [manpath], gz_cmd)
 
+# The two functions below are copied from IPython.utils.path, so we don't need
+# to import IPython during setup, which fails on Python 3.
+
+def target_outdated(target,deps):
+    """Determine whether a target is out of date.
+
+    target_outdated(target,deps) -> 1/0
+
+    deps: list of filenames which MUST exist.
+    target: single filename which may or may not exist.
+
+    If target doesn't exist or is older than any file listed in deps, return
+    true, otherwise return false.
+    """
+    try:
+        target_time = os.path.getmtime(target)
+    except os.error:
+        return 1
+    for dep in deps:
+        dep_time = os.path.getmtime(dep)
+        if dep_time > target_time:
+            #print "For target",target,"Dep failed:",dep # dbg
+            #print "times (dep,tar):",dep_time,target_time # dbg
+            return 1
+    return 0
+
+
+def target_update(target,deps,cmd):
+    """Update a target with a given command given a list of dependencies.
+
+    target_update(target,deps,cmd) -> runs cmd if target is outdated.
+
+    This is just a wrapper around target_outdated() which calls the given
+    command if target is outdated."""
+
+    if target_outdated(target,deps):
+        os.system(cmd)
+
 #---------------------------------------------------------------------------
 # Find scripts
 #---------------------------------------------------------------------------
@@ -277,9 +319,7 @@ def find_scripts(entry_points=False, suffix=''):
             'iptest%s = IPython.testing.iptest:main',
             'irunner%s = IPython.lib.irunner:main'
         ]]
-        gui_scripts = [s % suffix for s in [
-            'ipython%s-qtconsole = IPython.frontend.qt.console.qtconsoleapp:main',
-        ]]
+        gui_scripts = []
         scripts = dict(console_scripts=console_scripts, gui_scripts=gui_scripts)
     else:
         parallel_scripts = pjoin('IPython','parallel','scripts')
@@ -330,41 +370,12 @@ def check_for_dependencies():
 
 def record_commit_info(pkg_dir, build_cmd=build_py):
     """ Return extended build command class for recording commit
-
-    The extended command tries to run git to find the current commit, getting
-    the empty string if it fails.  It then writes the commit hash into a file
-    in the `pkg_dir` path, named ``.git_commit_info.ini``.
-
-    In due course this information can be used by the package after it is
-    installed, to tell you what commit it was installed from if known.
-
-    To make use of this system, you need a package with a .git_commit_info.ini
-    file - e.g. ``myproject/.git_commit_info.ini`` - that might well look like
-    this::
-
-        # This is an ini file that may contain information about the code state
-        [commit hash]
-        # The line below may contain a valid hash if it has been substituted
-        # during 'git archive'
-        archive_subst_hash=$Format:%h$
-        # This line may be modified by the install process
-        install_hash=
-
-    The .git_commit_info file above is also designed to be used with git
-    substitution - so you probably also want a ``.gitattributes`` file in the
-    root directory of your working tree that contains something like this::
-
-       myproject/.git_commit_info.ini export-subst
-
-    That will cause the ``.git_commit_info.ini`` file to get filled in by ``git
-    archive`` - useful in case someone makes such an archive - for example with
-    via the github 'download source' button.
-
-    Although all the above will work as is, you might consider having something
-    like a ``get_info()`` function in your package to display the commit
-    information at the terminal.  See the ``pkg_info.py`` module in the nipy
-    package for an example.
+    
+    records git commit in IPython.utils._sysinfo.commit
+    
+    for use in IPython.utils.sysinfo.sys_info() calls after installation.
     """
+    
     class MyBuildPy(build_cmd):
         ''' Subclass to write commit data into installation tree '''
         def run(self):
@@ -375,16 +386,12 @@ def record_commit_info(pkg_dir, build_cmd=build_py):
                                     stderr=subprocess.PIPE,
                                     shell=True)
             repo_commit, _ = proc.communicate()
+            repo_commit = repo_commit.strip()
             # We write the installation commit even if it's empty
-            cfg_parser = ConfigParser()
-            cfg_parser.read(pjoin(pkg_dir, '.git_commit_info.ini'))
-            if not cfg_parser.has_section('commit hash'):
-                # just in case the ini file is empty or doesn't exist, somehow
-                # we don't want the next line to raise
-                cfg_parser.add_section('commit hash')
-            cfg_parser.set('commit hash', 'install_hash', repo_commit.decode('ascii'))
-            out_pth = pjoin(self.build_lib, pkg_dir, '.git_commit_info.ini')
-            out_file = open(out_pth, 'wt')
-            cfg_parser.write(out_file)
-            out_file.close()
+            out_pth = pjoin(self.build_lib, pkg_dir, 'utils', '_sysinfo.py')
+            with open(out_pth, 'w') as out_file:
+                out_file.writelines([
+                    '# GENERATED BY setup.py\n',
+                    'commit = "%s"\n' % repo_commit.decode('ascii'),
+                ])
     return MyBuildPy
