@@ -1,4 +1,5 @@
 # encoding: utf-8
+from __future__ import print_function
 
 __docformat__ = "restructuredtext en"
 
@@ -27,26 +28,26 @@ def check_display(f):
 
 @check_display
 def print_line(char='='):
-    print char * 76
+    print(char * 76)
 
 @check_display
 def print_status(package, status):
     initial_indent = "%22s: " % package
     indent = ' ' * 24
-    print fill(str(status), width=76,
+    print(fill(str(status), width=76,
                initial_indent=initial_indent,
-               subsequent_indent=indent)
+               subsequent_indent=indent))
 
 @check_display
 def print_message(message):
     indent = ' ' * 24 + "* "
-    print fill(str(message), width=76,
+    print(fill(str(message), width=76,
                initial_indent=indent,
-               subsequent_indent=indent)
+               subsequent_indent=indent))
 
 @check_display
 def print_raw(section):
-    print section
+    print(section)
 
 #-------------------------------------------------------------------------------
 # Tests for specific packages
@@ -69,7 +70,7 @@ def check_for_sphinx():
         print_status('sphinx', "Not found (required for building documentation)")
         return False
     else:
-        print_status('sphinx', sphinx.__version__) 
+        print_status('sphinx', sphinx.__version__)
         return True
 
 def check_for_pygments():
@@ -136,29 +137,46 @@ def check_for_pyzmq():
     try:
         import zmq
     except ImportError:
-        print_status('pyzmq', "no (required for qtconsole and parallel computing capabilities)")
+        print_status('pyzmq', "no (required for qtconsole, notebook, and parallel computing capabilities)")
         return False
     else:
+        # pyzmq 2.1.10 adds pyzmq_version_info funtion for returning
+        # version as a tuple
+        if hasattr(zmq, 'pyzmq_version_info'):
+            if zmq.pyzmq_version_info() >= (2,1,4):
+                print_status("pyzmq", zmq.__version__)
+                return True
+            else:
+                # this branch can never occur, at least until we update our
+                # pyzmq dependency beyond 2.1.10
+                return False
+        # this is necessarily earlier than 2.1.10, so string comparison is
+        # okay
         if zmq.__version__ < '2.1.4':
             print_status('pyzmq', "no (have %s, but require >= 2.1.4 for"
             " qtconsole and parallel computing capabilities)"%zmq.__version__)
-            
+            return False
         else:
             print_status("pyzmq", zmq.__version__)
             return True
 
 def check_for_readline():
+    from distutils.version import LooseVersion
     try:
         import readline
     except ImportError:
         try:
             import pyreadline
-        except ImportError:
+            vs = pyreadline.release.version
+        except (ImportError, AttributeError):
             print_status('readline', "no (required for good interactive behavior)")
             return False
-        else:
-            print_status('readline', "yes pyreadline-"+pyreadline.release.version)
+        if LooseVersion(vs).version >= [1,7,1]:
+            print_status('readline', "yes pyreadline-" + vs)
             return True
+        else:
+            print_status('readline', "no pyreadline-%s < 1.7.1" % vs)
+            return False
     else:
         print_status('readline', "yes")
         return True
